@@ -85,6 +85,8 @@ extern const char* ltrsrv_get_client_cmd_str(int cmd);
 void ltrsrv_client_close(struct st_ltr_client *client) {
     int i, fnd;
 
+    ltrsrv_wait_remove_tmr(client);
+
     if (client->close_req && (client->close_req_err!=0) && (client->close_req_err!=LTRSRV_ERR_SOCKET_CLOSED)) {
         ltrsrv_log_rec(LTRSRV_LOGLVL_WARN, client->log_str, client->close_req_err, "Close client by error");
     } else {
@@ -129,9 +131,20 @@ void ltrsrv_client_close(struct st_ltr_client *client) {
     }
 }
 
+static void f_client_close_cb(t_socket sock, int event, void *data) {
+    t_ltr_client *client = (t_ltr_client *)data;
+    int err = _CLIENT_CHECK_HND(client);
+    if (!err) {
+        ltrsrv_client_close(client);
+    }
+}
+
 void ltrsrv_client_close_req(t_ltr_client *client, int err) {
-    client->close_req = 1;
-    client->close_req_err = err;
+    if (!client->close_req) {
+        client->close_req = 1;
+        client->close_req_err = err;
+        ltrsrv_wait_add_tmr(client, 0, 0, f_client_close_cb, client);
+    }
 }
 
 /* Создание строки для вывода сообщений о клиенте.
@@ -191,8 +204,9 @@ int p_client_create_logstr(t_ltr_client *client) {
                     else err = LTRSRV_ERR_MEMALLOC;
 
                     free(crate_str);
+                } else {
+                    err = LTRSRV_ERR_MEMALLOC;
                 }
-                else err = LTRSRV_ERR_MEMALLOC;
             } else {
                 client->log_str = (char*)malloc(sizeof(LTRSRV_LOGSRC_CLIENT) +
                                      strlen(client->location) + 5 +
@@ -494,9 +508,7 @@ int ltrsrv_client_put_words(struct st_ltr_client* client, const uint32_t* wrds,
 
     /* по ошибке передачи закрываем клиентское соединение */
     if (err) {
-        ltrsrv_log_rec(LTRSRV_LOGLVL_ERR, client->log_str, err,
-                       "Client socket data send error");
-        ltrsrv_client_close(client);
+        ltrsrv_client_close_req(client, err);
     }
 
     return err ? err :sent;
@@ -558,6 +570,10 @@ static void f_client_cb(t_socket sock, int event, void* data) {
             }
         }
 
+        /* при запросе на закрытые при обработке события от клиента сразу
+         * закрываем, не дожидаясь выполнения фонового закрытия.
+         * это позволяет сразу открыть клиента после закрытия без потенциальной
+         * ошибки из-за задержки фонового закрытия */
         if (client->close_req) {
             ltrsrv_client_close(client);
         }
@@ -656,16 +672,16 @@ static void f_accept_cb(t_socket sock, int event, void* data) {
 int ltrsrv_client_notify_mod_closed(struct st_ltr_client* client) {
     int err = _CLIENT_CHECK_HND(client);
     if (!err)
-        ltrsrv_client_close(client);
+        ltrsrv_client_close_req(client, 0);
     return err;
 }
 
 int ltrsrv_client_notify_crate_closed(t_ltr_crate* crate) {
     int i;
-    for (i=0; i < f_client_cnt; i++) {
+    for (i=f_client_cnt-1; i >=0; i--) {
         if (f_clients[i]->crate == crate) {
             f_clients[i]->crate = NULL;
-            ltrsrv_client_close(f_clients[i]);
+            ltrsrv_client_close_req(f_clients[i], 0);
         }
     }
     return 0;
