@@ -532,9 +532,11 @@ static int f_cmd_proc_init(t_ltr_client *client, int *resp_size, t_cmd_status *s
     /* если пришла первая команда инициализации - смотрим какой серйный
        номер указал клиент и к какому слоту он подключается */
     if (client->state == LTRSRV_CLIENT_STATE_INIT) {
-        const char* serial = (const char*)client->cmd->data;
+        const char* serial = (const char*)client->cmd->data;                
         uint8_t slot  =  client->cmd->data[LTR_CRATE_SERIAL_SIZE];
         uint8_t flags =  client->cmd->data[LTR_CRATE_SERIAL_SIZE+1];
+        /* завершающий нулевой символ в конце строки с серийным, на случай, если передан мусор */
+        client->cmd->data[LTR_CRATE_SERIAL_SIZE - 1] = 0;
 
         client->flags = flags;
 
@@ -996,7 +998,7 @@ static int f_cmd_proc_get_module_stat(t_ltr_client *client, int *presp_size, t_c
             if (crate!=NULL) {
                 t_ltr_module* module = &crate->modules[slot];
 
-                if (module->state == LTR_MODULE_STATE_WORK) {
+                if (module->state != LTR_MODULE_STATE_OFF) {
                     resp_size = sizeof(t_ltr_client_module_stat);
                     err = f_cmd_ex_resp_buf_realloc(client, resp_size);
                     if (!err) {
@@ -1009,13 +1011,14 @@ static int f_cmd_proc_get_module_stat(t_ltr_client *client, int *presp_size, t_c
                         mstat->snd_size = sizeof(t_ltr_client_module_stat);
                         mstat->flags = module->flags;
                         mstat->mid = CRATE_GET_MODULE_MID(crate, slot);
-                        strncpy(mstat->name, module->name, LTR_CRATE_MODULE_NAME_SIZE);
+                        strncpy(mstat->name, module->name, LTR_CRATE_MODULE_NAME_SIZE - 1);
+                        mstat->name[LTR_CRATE_MODULE_NAME_SIZE - 1] = '\0';
                         mstat->client_cnt = module->client_cnt;
                         mstat->stat = module->stat;
-                        mstat->stat.send_srvbuf_size = LTRSRV_MODULE_SEND_WORD_CNT;
-                        mstat->stat.send_srvbuf_full = module->snd_wrd_cnt;
-                        mstat->stat.recv_srvbuf_size = LTRSRV_MODULE_RECV_WORD_CNT;
-                        mstat->stat.recv_srvbuf_full = module->rcv_wrd_cnt;
+                        mstat->stat.send_srvbuf_size = CYCLEBUF_SIZE(&module->sndbuf);
+                        mstat->stat.send_srvbuf_full = CYCLEBUF_FILL_SIZE(&module->sndbuf);
+                        mstat->stat.recv_srvbuf_size = CYCLEBUF_SIZE(&module->rcvbuf);
+                        mstat->stat.recv_srvbuf_full = CYCLEBUF_FILL_SIZE(&module->rcvbuf);
                     }
                 } else {
                     err = LTRSRV_ERR_CRATE_SLOT_EMPTY;
@@ -1067,7 +1070,7 @@ static int f_cmd_proc_set_server_param(t_ltr_client *client, int *presp_size, t_
     err = f_cmd_ex_proc_hdr(client, 4, &cmd_data_size, NULL, &cmd_data);
     if (!err) {
         uint32_t param = CMD_GET_UINT32(cmd_data, 0);
-        err = ltrsrv_eth_set_param((t_ltr_server_params)param, &cmd_data[4], cmd_data_size-4);
+        err = ltrsrv_set_param((t_ltrd_params)param, &cmd_data[4], cmd_data_size-4);
     }
     return f_cmd_ex_set_resp(client, err, 0, presp_size, status);
 }
@@ -1082,7 +1085,7 @@ static int f_cmd_proc_get_server_param(t_ltr_client *client, int *presp_size, t_
         uint32_t param = CMD_GET_UINT32(cmd_data, 0);
         err = f_cmd_ex_resp_buf_realloc(client, resp_size);
         if (!err) {
-            err = ltrsrv_eth_get_param((t_ltr_server_params)param,
+            err = ltrsrv_get_param((t_ltrd_params)param,
                                    CMD_EX_RESP_DATA(client), &resp_size);
         }
     }
