@@ -21,10 +21,10 @@
 #define LTRSRV_SETTINGS_PARAM_LOG_LVL   "level"
 #define LTRSRV_SETTINGS_PARAM_LOG_FILE  "file"
 
-#define LTRSRV_SETTINGS_GROUP_CRATES_IP  "crate_ip_entries"
-#define LTRSRV_SETTINGS_PARAM_CRATE_IP   "ip"
-#define LTRSRV_SETTINGS_IP_ATTR_AUTOCON  "autocon"
-#define LTRSRV_SETTINGS_IP_ATTR_RECONNECT    "reconnect"
+#define LTRSRV_SETTINGS_GROUP_CRATES_IP     "crate_ip_entries"
+#define LTRSRV_SETTINGS_PARAM_CRATE_IP      "ip"
+#define LTRSRV_SETTINGS_IP_ATTR_AUTOCON     "autocon"
+#define LTRSRV_SETTINGS_IP_ATTR_RECONNECT   "reconnect"
 
 #define LTRSRV_SETTINGS_GROUP_INTF_ETH_PAR  "intf_eth_params"
 #define LTRSRV_SETTINGS_PARAM_ETH_POLL_TIME "crate_poll_time"
@@ -33,6 +33,12 @@
 #define LTRSRV_SETTINGS_PARAM_ETH_CTLCMD_TIMEOUT "crate_ctlcmd_timeout"
 #define LTRSRV_SETTINGS_PARAM_ETH_INTF_CHECK_TIME "intf_check_time"
 #define LTRSRV_SETTINGS_PARAM_ETH_RECONNECT_TIME "crate_reconnect_time"
+#define LTRSRV_SETTINGS_PARAM_ETH_DATA_NODELAY    "tcp_data_nodelay"
+
+
+#define LTRSRV_SETTINGS_GROUP_CRATE_PARAMS                "crate_params"
+#define LTRSRV_SETTINGS_PARAM_CRATE_MODULE_SEND_BUF_SIZE   "module_send_buf_size"
+#define LTRSRV_SETTINGS_PARAM_CRATE_MODULE_RECV_BUF_SIZE   "module_recv_buf_size"
 
 static int f_modified = 0;
 static const char* f_filename=NULL;
@@ -287,10 +293,10 @@ static void f_load_log(xmlNode *log_group) {
     for (;param; param=param->next) {
         if (param->type == XML_ELEMENT_NODE) {
             xmlChar* val = f_get_node_text(param);
-            if (!strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_LOG_LVL)) {
+            if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_LOG_LVL)) {
                 if (val)
                     ltrsrv_log_set_lvl((t_ltrsrv_log_level)atoi((char*)val), 1, 0);
-            } else if (!strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_LOG_FILE)) {
+            } else if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_LOG_FILE)) {
                 if (val)
                     ltrsrv_log_set_file((char*)f_trim_str(val));
             }
@@ -302,7 +308,7 @@ static void f_load_crates_ip(xmlNode *ip_group) {
     xmlNode *ip = ip_group->children;
     for (;ip; ip=ip->next) {
         if ((ip->type == XML_ELEMENT_NODE) &&
-                !strcmp((char*)ip->name, LTRSRV_SETTINGS_PARAM_CRATE_IP)) {
+                !strcmp((const char*)ip->name, LTRSRV_SETTINGS_PARAM_CRATE_IP)) {
             xmlChar* ip_val = f_trim_str(f_get_node_text(ip));
             uint32_t addr;
             uint32_t flags = 0;
@@ -310,13 +316,13 @@ static void f_load_crates_ip(xmlNode *ip_group) {
             if (inet_pton(AF_INET, (char*)ip_val, &addr)==1) {
                 xmlAttr* attr;
                 for (attr = ip->properties; attr; attr=attr->next) {
-                    if (!strcmp((char*)attr->name, LTRSRV_SETTINGS_IP_ATTR_AUTOCON)) {
-                        int val = atoi((char*)attr->children->content);
+                    if (!strcmp((const char*)attr->name, LTRSRV_SETTINGS_IP_ATTR_AUTOCON)) {
+                        int val = atoi((const char*)attr->children->content);
                         if (val) {
                             flags |= LTR_CRATE_IP_FLAG_AUTOCONNECT;
                         }
-                    } else if (!strcmp((char*)attr->name, LTRSRV_SETTINGS_IP_ATTR_RECONNECT)) {
-                        int val = atoi((char*)attr->children->content);
+                    } else if (!strcmp((const char*)attr->name, LTRSRV_SETTINGS_IP_ATTR_RECONNECT)) {
+                        int val = atoi((const char*)attr->children->content);
                         if (val) {
                             flags |= LTR_CRATE_IP_FLAG_RECONNECT;
                         }
@@ -336,22 +342,44 @@ static void f_load_intf_eth(xmlNode *eth_group) {
         if (param->type == XML_ELEMENT_NODE) {
             xmlChar* val = f_get_node_text(param);
 
-            if (!strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_POLL_TIME)) {
-                uint32_t setval = atoi((char*)val);
-                ltrsrv_eth_set_param(LTR_SERVER_PARAM_ETH_CRATE_POLL_TIME, &setval, sizeof(setval));
-            } else if (!strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_CON_TIMEOUT) ||
-                       !strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_CON_TIMEOUT_OLD)) {
-                uint32_t setval = atoi((char*)val);
-                ltrsrv_eth_set_param(LTR_SERVER_PARAM_ETH_CRATE_CON_TOUT, &setval, sizeof(setval));
-            } else if (!strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_CTLCMD_TIMEOUT)) {
-                uint32_t setval = atoi((char*)val);
-                ltrsrv_eth_set_param(LTR_SERVER_PARAM_ETH_CRATE_CTLCMD_TOUT, &setval, sizeof(setval));
-            } else if (!strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_INTF_CHECK_TIME)) {
-                uint32_t setval = atoi((char*)val);
-                ltrsrv_eth_set_param(LTR_SERVER_PARAM_ETH_INTF_CHECK_TIME, &setval, sizeof(setval));
-            } else if (!strcmp((char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_RECONNECT_TIME)) {
-                uint32_t setval = atoi((char*)val);
-                ltrsrv_eth_set_param(LTR_SERVER_PARAM_ETH_RECONNECTION_TIME, &setval, sizeof(setval));
+            if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_POLL_TIME)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_ETH_CRATE_POLL_TIME, &setval, sizeof(setval));
+            } else if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_CON_TIMEOUT) ||
+                       !strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_CON_TIMEOUT_OLD)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_ETH_CRATE_CON_TOUT, &setval, sizeof(setval));
+            } else if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_CTLCMD_TIMEOUT)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_ETH_CRATE_CTLCMD_TOUT, &setval, sizeof(setval));
+            } else if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_INTF_CHECK_TIME)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_ETH_INTF_CHECK_TIME, &setval, sizeof(setval));
+            } else if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_RECONNECT_TIME)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_ETH_RECONNECTION_TIME, &setval, sizeof(setval));
+            } else if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_ETH_DATA_NODELAY)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_ETH_SEND_NODELAY, &setval, sizeof(setval));
+            }
+        }
+    }
+}
+
+
+static void f_load_crate_params(xmlNode *eth_group) {
+    xmlNode *param = eth_group->children;
+
+    for (;param; param=param->next) {
+        if (param->type == XML_ELEMENT_NODE) {
+            xmlChar* val = f_get_node_text(param);
+
+            if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_CRATE_MODULE_SEND_BUF_SIZE)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_MODULE_SEND_BUF_SIZE, &setval, sizeof(setval));
+            } else if (!strcmp((const char*)param->name, LTRSRV_SETTINGS_PARAM_CRATE_MODULE_RECV_BUF_SIZE)) {
+                uint32_t setval = (uint32_t)atoi((const char*)val);
+                ltrsrv_set_param(LTRD_PARAM_MODULE_RECV_BUF_SIZE, &setval, sizeof(setval));
             }
         }
     }
@@ -376,7 +404,7 @@ int ltrsrv_load_settings(const char* filename) {
     /* получаем root-элемент и проверяем его имя */
     if (!err) {
         root = xmlDocGetRootElement(doc);
-        if (strcmp((char*)root->name, LTRSRV_SETTINGS_ROOT_ELEM_NAME))
+        if (strcmp((const char*)root->name, LTRSRV_SETTINGS_ROOT_ELEM_NAME))
             err = LTRSRV_ERR_CFGFILE_ROOT_ELEM;
     }
 
@@ -388,12 +416,14 @@ int ltrsrv_load_settings(const char* filename) {
 
         for (; settings_group; settings_group = settings_group->next) {
             if (settings_group->type == XML_ELEMENT_NODE) {
-                if (!strcmp((char*)settings_group->name, LTRSRV_SETTINGS_GROUP_LOG)) {
+                if (!strcmp((const char*)settings_group->name, LTRSRV_SETTINGS_GROUP_LOG)) {
                     f_load_log(settings_group);
-                } else if (!strcmp((char*)settings_group->name, LTRSRV_SETTINGS_GROUP_CRATES_IP)) {
+                } else if (!strcmp((const char*)settings_group->name, LTRSRV_SETTINGS_GROUP_CRATES_IP)) {
                     f_load_crates_ip(settings_group);
-                } else if (!strcmp((char*)settings_group->name, LTRSRV_SETTINGS_GROUP_INTF_ETH_PAR)) {
+                } else if (!strcmp((const char*)settings_group->name, LTRSRV_SETTINGS_GROUP_INTF_ETH_PAR)) {
                     f_load_intf_eth(settings_group);
+                } else if (!strcmp((const char*)settings_group->name, LTRSRV_SETTINGS_GROUP_CRATE_PARAMS)) {
+                    f_load_crate_params(settings_group);
                 }
             }
         }
@@ -506,45 +536,79 @@ int ltrsrv_save_settings(void) {
         }
 
         if (!err) {
-            xmlNodePtr log_node = xmlNewChild(root, NULL, BAD_CAST LTRSRV_SETTINGS_GROUP_INTF_ETH_PAR, NULL);
-            if (log_node!=NULL) {
+            xmlNodePtr eth_node = xmlNewChild(root, NULL, BAD_CAST LTRSRV_SETTINGS_GROUP_INTF_ETH_PAR, NULL);
+            if (eth_node!=NULL) {
                 uint32_t val;
                 uint32_t size = sizeof(val);
 
-                err = ltrsrv_eth_get_param(LTR_SERVER_PARAM_ETH_CRATE_POLL_TIME, &val, &size);
+                err = ltrsrv_get_param(LTRD_PARAM_ETH_CRATE_POLL_TIME, &val, &size);
                 if (!err) {
                     sprintf(tmp_buf, "%u", val);
-                    xmlNewChild(log_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_POLL_TIME, BAD_CAST tmp_buf);
+                    xmlNewChild(eth_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_POLL_TIME, BAD_CAST tmp_buf);
                 }
 
                 if (!err)
-                    err = ltrsrv_eth_get_param(LTR_SERVER_PARAM_ETH_CRATE_CON_TOUT, &val, &size);
+                    err = ltrsrv_get_param(LTRD_PARAM_ETH_CRATE_CON_TOUT, &val, &size);
                 if (!err) {
                     sprintf(tmp_buf, "%u", val);
-                    xmlNewChild(log_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_CON_TIMEOUT, BAD_CAST tmp_buf);
+                    xmlNewChild(eth_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_CON_TIMEOUT, BAD_CAST tmp_buf);
                 }
 
                 if (!err)
-                    err = ltrsrv_eth_get_param(LTR_SERVER_PARAM_ETH_CRATE_CTLCMD_TOUT, &val, &size);
+                    err = ltrsrv_get_param(LTRD_PARAM_ETH_CRATE_CTLCMD_TOUT, &val, &size);
                 if (!err) {
                     sprintf(tmp_buf, "%u", val);
-                    xmlNewChild(log_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_CTLCMD_TIMEOUT, BAD_CAST tmp_buf);
+                    xmlNewChild(eth_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_CTLCMD_TIMEOUT, BAD_CAST tmp_buf);
                 }
 
 
                 if (!err)
-                    err = ltrsrv_eth_get_param(LTR_SERVER_PARAM_ETH_INTF_CHECK_TIME, &val, &size);
+                    err = ltrsrv_get_param(LTRD_PARAM_ETH_INTF_CHECK_TIME, &val, &size);
                 if (!err) {
                     sprintf(tmp_buf, "%u", val);
-                    xmlNewChild(log_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_INTF_CHECK_TIME, BAD_CAST tmp_buf);
+                    xmlNewChild(eth_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_INTF_CHECK_TIME, BAD_CAST tmp_buf);
                 }
 
 
                 if (!err)
-                    err = ltrsrv_eth_get_param(LTR_SERVER_PARAM_ETH_RECONNECTION_TIME, &val, &size);
+                    err = ltrsrv_get_param(LTRD_PARAM_ETH_RECONNECTION_TIME, &val, &size);
                 if (!err) {
                     sprintf(tmp_buf, "%u", val);
-                    xmlNewChild(log_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_RECONNECT_TIME, BAD_CAST tmp_buf);
+                    xmlNewChild(eth_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_RECONNECT_TIME, BAD_CAST tmp_buf);
+                }
+
+                if (!err)
+                    err = ltrsrv_get_param(LTRD_PARAM_ETH_SEND_NODELAY, &val, &size);
+                if (!err) {
+                    sprintf(tmp_buf, "%u", val);
+                    xmlNewChild(eth_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_ETH_DATA_NODELAY, BAD_CAST tmp_buf);
+                }
+
+
+
+            } else {
+                err = LTRSRV_ERR_SAVECFG_XML_NODE;
+            }
+        }
+
+
+        if (!err) {
+            xmlNodePtr crate_par_node = xmlNewChild(root, NULL, BAD_CAST LTRSRV_SETTINGS_GROUP_CRATE_PARAMS, NULL);
+            if (crate_par_node!=NULL) {
+                uint32_t val;
+                uint32_t size = sizeof(val);
+
+                err = ltrsrv_get_param(LTRD_PARAM_MODULE_SEND_BUF_SIZE, &val, &size);
+                if (!err) {
+                    sprintf(tmp_buf, "%u", val);
+                    xmlNewChild(crate_par_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_CRATE_MODULE_SEND_BUF_SIZE, BAD_CAST tmp_buf);
+                }
+
+                if (!err)
+                    err = ltrsrv_get_param(LTRD_PARAM_MODULE_RECV_BUF_SIZE, &val, &size);
+                if (!err) {
+                    sprintf(tmp_buf, "%u", val);
+                    xmlNewChild(crate_par_node, NULL, BAD_CAST LTRSRV_SETTINGS_PARAM_CRATE_MODULE_RECV_BUF_SIZE, BAD_CAST tmp_buf);
                 }
             } else {
                 err = LTRSRV_ERR_SAVECFG_XML_NODE;
@@ -571,8 +635,10 @@ int ltrsrv_save_settings(void) {
          */
         xmlMemoryDump();
 
-        if (!err)
-            f_modified = 0;
+        /* сбрасываем в 0 даже при ошибке, иначе можем уйти в бесконечный
+         * цикл при отсутствии прав сохранения.
+         * @todo подумать о планировании сохранить через заданное время */
+        f_modified = 0;
     }
     return err;
 }
