@@ -4,7 +4,8 @@
 #include "config.h"
 #include "ltr_crate_defs.h"
 #include "ltimer.h"
-
+#include "ltrsrv_cyclebuf.h"
+#include "ltrsrv_params.h"
 
 #include <time.h>
 
@@ -47,6 +48,7 @@
 
 
 #define LTR_CRATE_SUPPORT_PROTOCOL_VER(crate)  (crate->descr.protocol_ver_major || crate->descr.protocol_ver_minor)
+#define LTR_CRATE_SUPPORT_SEC_CTL_CON(crate)  (crate->descr.protocol_ver_major || crate->descr.protocol_ver_minor)
 #define LTR_CRATE_SUPPORT_SLOTS_CONFIG(crate)  (crate->descr.slots_config_ver_major >=2)
 
 //typedef struct st_ltr_crate t_ltr_crate;
@@ -124,15 +126,17 @@ typedef struct {
 
 
 typedef enum {
-    LTR_MODULE_STATE_OFF = 0,
-    LTR_MODULE_STATE_NEED_RST = 1,
-    LTR_MODULE_STATE_INIT = 2,
-    LTR_MODULE_STATE_CHECK_CFG = 3,
-    LTR_MODULE_STATE_WORK = 4
+    LTR_MODULE_STATE_OFF = 0,      /* модуль отсутствует в слоте (не было признака наличия модуля) */
+    LTR_MODULE_STATE_NEED_RST = 1, /* для модуля нужно послать STOP-STOP-RESET-STOP и выполнить переинициализацию */
+    LTR_MODULE_STATE_INIT = 2,     /* послана команда STOP-STOP-RESET-STOP, но еще не получен ответ */
+    LTR_MODULE_STATE_CHECK_CFG = 3, /* для модуля необходимо выполнить проверку наличия сохраненной в крейте информации о типе модуля (kdr extenstion) */
+    LTR_MODULE_STATE_WORK = 4, /* модуль в рабочем режиме */
+    LTR_MODULE_STATE_NEED_GET_SUBID = 5, /* для завершения инициализации модуля необходимо дополнительно запросить SubID */
+    LTR_MODULE_STATE_WAIT_SUBID = 6, /* послан запрос на получения subid, ожидмется ответ от модуля */
 } t_ltr_module_state;
 
 typedef enum {
-    LTR_MODULE_FLAGS_HIGH_BAUD             = 0x001, /** признак, что модуль использует высокую скорость */
+    LTR_MODULE_FLAGS_HIGH_BAUD            = 0x001, /** признак, что модуль использует высокую скорость */
     LTR_MODULE_FLAGS_USE_SND_FIFO_STATUS  = 0x100, /** признак, что модуль использует статистику FIFO на передачу данных */
     LTR_MODULE_FLAGS_USE_SYNC_MARK        = 0x200 /** признак, что модуль поддерживает генерацию синхрометок */
 } t_ltr_module_flags;
@@ -151,6 +155,21 @@ typedef enum {
     LTR_CRATE_CMD_CODE_IOCTL = 2 /**< специальный запрос ввода-вывода */
 } t_crate_cmd_code;
 
+/** смысл команды - причина ее вызова */
+typedef enum {
+    LTR_CRATE_CMD_REASON_GET_MODULES     = 0, /**< чтение состова модулей */
+    LTR_CRATE_CMD_REASON_SYSFAULT        = 1, /**< чтение регистров при ошибке крейта */
+    LTR_CRATE_CMD_REASON_BAUD_CH         = 2, /**< запись новой маски скорости */
+    LTR_CRATE_CMD_REASON_CLIENT_REQ      = 3, /**< запрос от клиента */
+    LTR_CRATE_CMD_REASON_SLOT_CFG        = 4, /**< проверка конфигурации слотов */
+    LTR_CRATE_CMD_REASON_CRATE_POLL      = 5, /**< периодический опрос состояния крейта */
+    LTR_CRATE_CMD_REASON_INTF_REQ        = 6  /**< запрос со стороны интерфейса */
+} t_ltr_crate_cmd_reason;
+
+typedef enum {
+    LTR_CRATE_CMD_FLAGS_EXTERNAL_BUF = 1,
+} t_ltr_crate_cmd_flags;
+
 typedef enum {
     LTR_CRATE_DESCR_FLAGS_SUPPORT_BOARD_REV = 0x0001, /**< Поддержка запроса на получение ревизии */
     LTR_CRATE_DESCR_FLAGS_SUPPORT_POLL_CMD  = 0x0002, /**< Поддержка команды опроса состояния крейта */
@@ -203,6 +222,8 @@ typedef int (*t_ltr_module_proc_wrd_func)(struct st_ltr_crate *crate, struct st_
 /* функция для очистки памяти, для специфичной для модуля обработки */
 typedef void (*t_ltr_module_spec_free)(struct st_ltr_crate *crate, struct st_ltr_module *module);
 
+typedef void (*t_ltr_module_subid_req)(struct st_ltr_crate *crate, struct st_ltr_module *module);
+
 
 /** callback-функция, вызываемая после завершения обработки команды интерфейсом.
     @param[in] hnd  Описатель крейта
@@ -290,36 +311,37 @@ typedef struct {
     t_ltr_module_spec_free     free; /* освобождение памяти под специфичные для модуля данные */
 } t_ltr_module_spec_func;
 
+
+
+
+
+
 /** структура, описывающая состояние модуля */
 typedef struct st_ltr_module {
+    struct st_ltr_crate *crate;
     char name[LTR_CRATE_MODULE_NAME_SIZE]; /* название модуля */
     char *log_str; /* строка для вывода сообщений в журнал */
     t_ltr_module_state state; /** состояние модуля */
+    int rst_retry_cnt; /*  количество повторных выполнений операции сброса модуля */
+    int subid_retry_cnt; /* количество повторных запросов расширенного id модуля */
     int flags; /** флаги описывающие модуль из t_ltr_module_flags */
     int state_flags; /** флаги состояния модуля t_ltr_module_state_flags */
     t_ltr_module_spec_func func; /** функции, специфичные для модуля */
     void *spec_data; /* указатель для сохранения данных, специфичных для модуля */
-    struct st_ltr_client* clients[LTRSRV_CLIENT_PER_MODULE_MAX]; /** список клиентов */
+    struct st_ltr_client *clients[LTRSRV_CLIENT_PER_MODULE_MAX]; /** список клиентов */
     //int clients_sent_pos[LTRSRV_CLIENT_PER_MODULE_MAX]; /** позиция в буфере модуля, откуда будем передавать клиенту */
     int client_ovfl[LTRSRV_CLIENT_PER_MODULE_MAX]; /** признак, что этот клиент не успел за остальными */
     uint16_t client_cnt; /** количество подключенных клиентов */
     uint16_t mid; /** id-модуля */
+    uint16_t subid_code; /** subdi-модуля (ltr01) */
     t_ltr_module_stat stat; /** статистика модуля */
 
     uint64_t bw_last_wrd_sent; /** Количество переданных слов при последней проверке скорости передачи */
     uint64_t bw_last_wrd_recv; /** Количество принятых слов при последней проверке скорости передачи */
 
-    /* параметры буфера с данными для передачи модулю */
-    uint32_t snd_put_pos;
-    uint32_t snd_sent_pos;
-    uint32_t snd_wrd_cnt;
-    uint32_t *snd_buf;
 
-    /* параметры буфера для принятых данных от модуля */
-    uint32_t rcv_put_pos;
-    uint32_t rcv_sent_pos;
-    uint32_t rcv_wrd_cnt;
-    uint32_t *rcv_buf;
+    t_cycle_buf sndbuf; /* параметры буфера с данными для передачи модулю */
+    t_cycle_buf rcvbuf; /* параметры буфера для принятых данных от модуля */
 } t_ltr_module;
 
 
@@ -376,6 +398,7 @@ typedef struct st_ltr_crate {
 
 
 
+
 int ltrsrv_crate_register(t_ltr_crate *crate);
 int ltrsrv_crate_unregister(t_ltr_crate *crate);
 
@@ -398,12 +421,15 @@ int ltrsrv_crate_unregister_client(t_ltr_crate *crate, int slot, struct st_ltr_c
 int ltrsrv_notify_client_data_rdy(struct st_ltr_client *client, t_ltr_crate *crate, int slot);
 
 
-t_ltr_crate* ltrsrv_crate_find(t_ltr_crate_interface intf, const char *serial);
+t_ltr_crate *ltrsrv_crate_find(t_ltr_crate_interface intf, const char *serial);
+
 int ltrsrv_get_crates(char serial_list[][LTR_CRATE_SERIAL_SIZE], int max_cnt, int *fnd_cnt);
 int ltrsrv_get_crates_info(t_crate_info_entry *info_list, int max_cnt, int flags, int *fnd_cnt);
 int ltrsrv_get_crate_modules(t_ltr_crate *crate, uint16_t *mids, int max_mids);
 int ltrsrv_crate_start_client_cmd(t_ltr_crate *crate, t_crate_cmd_code cmd, uint32_t req, uint32_t param,
                                   uint8_t *buf, uint32_t snd_size, uint32_t *recvd_size, t_ltr_crate_cmd_cb cb, void *cb_data);
+int ltrsrv_crates_exec_for_all(int (*exec_funt)(t_ltr_crate *crate, void *data), void *data);
+
 
 
 int ltrsrv_crate_reset_module(t_ltr_crate *crate, int slot, int flags);
@@ -415,4 +441,10 @@ int ltrsrv_crate_init_parse_fpga_data(const char *data, t_ltr_crate_descr *descr
 
 int ltrsrv_crate_data_channel_reset(t_ltr_crate *crate);
 
+int ltrsrv_crate_add_cmd_to_queue(t_ltr_crate *hnd, t_crate_cmd_code code, t_ltr_crate_cmd_reason reason,
+                                 uint32_t req, uint32_t param,
+                                 uint32_t snd_size, uint32_t rcv_size, uint32_t *recvd_size,
+                                 uint8_t *data, int flags, t_ltr_crate_cmd_cb cb, void *cb_data);
+
+
 #endif
