@@ -103,6 +103,7 @@ typedef struct {
 
 
 static uint8_t f_rcv_buf[LTRSRV_ETH_RCV_BUF_SIZE];
+static int f_eth_nodelay = 0;
 
 
 #define ETH_CTL_REQ_HDR_SIZE (offsetof(t_eth_ctl_req, data))
@@ -126,8 +127,8 @@ typedef struct {
     int st_flags; /* флаги из t_ip_entry_stflags */
     t_ltimer recon_tmr; /* таймер для переподключения */
     unsigned recon_cnt;
-    t_ltr_crate* crate;
     t_ltr_crate_ip_status status;
+    t_ltr_crate *crate;
 } t_ltr_crate_ip_entry;
 
 /* состояние автомата инициализации крейта */
@@ -148,7 +149,9 @@ typedef enum {
 
 /* дополнительные флаги о состоянии интерфейса */
 typedef enum {
-    _ETH_FLAGS_DATA_SOCK_RECONNECTION = 1 /* идет повторное установление связи по данным */
+    ETH_FLAGS_DATA_SOCK_RECONNECTION   = 1 << 0, /* идет повторное установление связи по данным */
+    ETH_FLAGS_DATA_SOCK_CHECK_SEC_CON  = 1 << 1, /* проверка, не закрыто ли соединение данных из-за
+                                                    перевода соединения во вторичное состояние */
 } t_eth_flags;
 
 
@@ -198,6 +201,12 @@ typedef struct {
     int rcv_last_wrd; /* сюда сохраняется не кратный 4-м хвост */
     int data_recon_cnt; /* количество попыток повторить соединение */
 
+    /* данные команды, выполняемой в рабочем режиме через очередь команд */
+    struct {
+        uint32_t recvd_size;
+        uint8_t  recvd_data[32];
+    } work_cmd_ctx;
+
 #ifdef LTRD_ETH_DUMP_ENABLED
     uint32_t dump_pos;
     t_dump_packet dump[DUMP_PACKET_MAX_SIZE];
@@ -212,6 +221,9 @@ static void f_check_intf(void);
 static void f_intf_check_tout_cb(t_socket sock, int event, void *data);
 static void f_check_recon_cb(t_socket sock, int event, void *data);
 
+static int  f_start_data_recon(t_ltr_crate *crate);
+static void f_data_reconnection_finish(t_ltr_crate *crate);
+
 
 static t_ltr_crate_ip_entry f_ip_entries[LTRSRV_ETH_STATIC_IP_ENTRY_CNT];
 static int f_ip_cnt = 0;
@@ -244,6 +256,15 @@ static t_ltrsrv_str_tbl f_ethcmd_str[] = {
 #define LOGSTR_CMD_CODE(cmd) ltrsrv_log_get_str(cmd, f_ethcmd_str, sizeof(f_ethcmd_str)/sizeof(f_ethcmd_str[0]))
 
 
+static int f_sock_set_nodelay(t_socket s, int nodelay) {
+    int err = LTRSRV_ERR_SUCCESS;
+    if (L_SOCKET_ERROR == setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (char*)&nodelay, sizeof(nodelay)))
+        err = LTRSRV_ERR_SOCKET_SET_NODELAY;
+    return err;
+}
+
+
+
 
 /******************************************************************************
   Функция запускает подключение по сокету.
@@ -289,6 +310,10 @@ static int f_sock_start_conn(t_socket* psock, int family, const char* ip_addr,
     }
 #endif
 
+    if (!err && f_eth_nodelay) {
+        err = f_sock_set_nodelay(s, 1);
+    }
+
     if (!err) {
         /* заполняем структуру с адресом LTR-сервера */
         memset(&peer, 0, sizeof(peer));
@@ -474,6 +499,13 @@ static int f_start_get_descr(t_ltr_crate *crate) {
                       sizeof(t_ltr_device_raw_descr), 0, NULL);
 }
 
+static int f_start_get_primary_con_status(t_ltr_crate *crate) {
+    return f_start_ctl_req(crate, LTR_ETH_CMD_GET_ARRAY,
+                        LTR_CRATE_ADDR_CONNECTION_PRIMARY,
+                        LTR_CRATE_REG_CONNECTION_PRIMARY_SIZE,
+                        0, NULL);
+}
+
 
 static void f_print_connection_err(t_ltr_crate *crate, int err, const char *str) {
     t_ltr_crate_ip_entry *ip_entry = f_get_crate_ipentry(crate);
@@ -486,6 +518,49 @@ static void f_print_connection_err(t_ltr_crate *crate, int err, const char *str)
 }
 
 
+
+
+static int f_check_sec_con_data_close(struct st_ltr_crate *crate, int err, void *prv_data) {
+    (void)prv_data;
+
+    /* Если вместе с сокетом данных крейт закрыл корректно и управляющее соединение, то
+     * считаем, что это соответстует закрытию соединения из-за установки соединения
+     * другим клиентом. */
+    /** @todo возможно стоит попробовать заново утсановить контрольное соединение
+     * только для проверки наличия второго */
+    /** @todo посылка специального кода исключения в новых версиях прошивок? */
+    if (err == LTRSRV_ERR_SOCKET_CLOSED) {
+        err = 0;
+        ltrsrv_log_rec(LTRSRV_LOGLVL_WARN, crate->log_str, err,
+                       "Crate close data and control sockets. Another host can establish primary connection...");
+        ltrsrv_crate_close_req(crate, 0);
+    } else if (err == LTRSRV_ERR_SUCCESS) {
+        t_eth_data *intf = (t_eth_data *)crate->intf_data;
+        int is_primary =0;
+
+        intf->flags &= ~ETH_FLAGS_DATA_SOCK_CHECK_SEC_CON;
+
+        if (intf->work_cmd_ctx.recvd_size > 0) {
+            is_primary = intf->work_cmd_ctx.recvd_data[0];
+        }
+
+        if (!is_primary) {
+            ltrsrv_log_rec(LTRSRV_LOGLVL_WARN, crate->log_str, 0,
+                           "Crate change connection status to secondary. Another host established crate primary connection.");
+        } else {
+            err = f_start_data_recon(crate);
+        }
+    }
+
+    if (err) {
+        ltrsrv_log_rec(LTRSRV_LOGLVL_ERR, crate->log_str, err,
+                       "Crate close data socket. Reconnection attemt error");
+        ltrsrv_crate_close_req(crate, err);
+    }
+    return  err;
+}
+
+
 /****************************************************************************//**
   Функция вызывается после того, как команда была передана и получен весь ответ.
     Переданная команда находится в st->req, а ответ в st->reply.
@@ -556,12 +631,8 @@ static int f_process_cmd(t_ltr_crate* crate) {
                 if (!err) {
                     /* при наличии версии протокола - проверяем,
                      * первичное ли это подключение */
-                    if (LTR_CRATE_SUPPORT_PROTOCOL_VER(crate)) {
-                        err = f_start_ctl_req(crate, LTR_ETH_CMD_GET_ARRAY,
-                                            LTR_CRATE_ADDR_CONNECTION_PRIMARY,
-                                            LTR_CRATE_REG_CONNECTION_PRIMARY_SIZE,
-                                            0, NULL);
-                        break;
+                    if (LTR_CRATE_SUPPORT_SEC_CTL_CON(crate)) {
+                        err = f_start_get_primary_con_status(crate);
                     } else {
                         /* иначе переходим к получению имени крейта */
                         err = f_start_get_name(crate);
@@ -584,20 +655,22 @@ static int f_process_cmd(t_ltr_crate* crate) {
                 err = ltrsrv_crate_init_process_data(crate, intf->req->CmdParam, intf->reply->data, intf->reply->VarDataLen);
                 if (!err) {
                     switch (intf->req->CmdParam) {
-                        //нет break!
                         case LTR_CRATE_ADDR_CONNECTION_PRIMARY: {
                                 uint8_t is_primary = intf->reply->data[0];
                                 if (!is_primary) {
                                     ltrsrv_log_rec(LTRSRV_LOGLVL_WARN, crate->log_str, 0,
                                                    "Connection already established! Force primary connection!");
-
-                                    err = f_start_ctl_req(crate, LTR_ETH_CMD_PUT_ARRAY,
-                                                        LTR_CRATE_ADDR_CONNECTION_FORCE,
-                                                        0, 0, NULL);
-                                    break;
                                 }
+                                /* Всегда делаем сброс соединения данных, даже если текущее
+                                 * соединение - primary. Это ничему не вредит, но
+                                 * может помочь с глюком LTR-CEU-1 с прошивками до 1.0.6,
+                                 * т.к. в нем могло оставаться старое возможно некоректно
+                                 * оборванное соединение по primary-подключению */
+                                err = f_start_ctl_req(crate, LTR_ETH_CMD_PUT_ARRAY,
+                                                    LTR_CRATE_ADDR_CONNECTION_FORCE,
+                                                    0, 0, NULL);
                             }
-                            //нет break!
+                            break;
                         case LTR_CRATE_ADDR_CONNECTION_FORCE:
                             /* если есть расширенная версия протокола, то считаем, что
                              * должны поддерживать возможность получить версию
@@ -633,7 +706,7 @@ static int f_process_cmd(t_ltr_crate* crate) {
                                                    0, NULL);
                                 break;
                             }
-                            //нет break!
+                            /* fallthrough */
                         case LTR_CRATE_ADDR_BOARD_REV:
                             intf->state = ETH_INIT_STATE_DATA_CH_CONN;
                             break;
@@ -826,11 +899,34 @@ static void f_save_dump(t_ltr_crate *crate) {
 #endif
 
 
+static int f_start_data_recon(t_ltr_crate *crate) {
+    int err = 0;
+    int wt = 0;
+    t_eth_data *intf = (t_eth_data *)crate->intf_data;
+    if (++intf->data_recon_cnt <= LTR_TCP_DATA_RECON_LIMIT) {
+        ltrsrv_log_rec(LTRSRV_LOGLVL_WARN, crate->log_str, 0,
+                       "Data socket closed by crate. Start data socket reconnection (reconnection number = %d)!",
+                       intf->data_recon_cnt);
+        err = f_sock_start_conn(&intf->data_sock, AF_INET, crate->par.location,
+                              LTR_TCP_DATA_PORT, &wt, crate, f_data_cb);
+        if (!err) {
+            if (wt) {
+                intf->flags |= ETH_FLAGS_DATA_SOCK_RECONNECTION;
+            } else {
+                f_data_reconnection_finish(crate);
+            }
+        }
+    } else {
+        err = LTRSRV_ERR_CRATE_DATA_RECON_EXCEEDED;
+    }
+    return  err;
+}
+
 static void f_data_reconnection_finish(t_ltr_crate *crate) {
     t_eth_data *intf = (t_eth_data *)crate->intf_data;
     int err;
 
-    intf->flags &= ~_ETH_FLAGS_DATA_SOCK_RECONNECTION;
+    intf->flags &= ~ETH_FLAGS_DATA_SOCK_RECONNECTION;
     err = ltrsrv_wait_add_sock(intf->data_sock, LTRSRV_WAITFLG_RECV, 0, f_data_cb, crate);
     if (!err) {
         ltrsrv_log_rec(LTRSRV_LOGLVL_INFO, crate->log_str, err, "Data socket reconnection done");
@@ -872,35 +968,29 @@ static void f_data_cb(t_socket sock, int event, void* data) {
                установить соединение заново. Однако ограничиваем число
                попыток, чтобы бесконечно не переоткрывать */
 
-            /** @todo Сделать проверку (если поддерживается крейтом), не
-             *        перебили ли наше соединение */
-            int wt = 0;
-
             intf->rcv_data_offs = 0;
             ltrsrv_wait_remove_sock(intf->data_sock);
             closesocket(intf->data_sock);
             intf->data_sock = L_INVALID_SOCKET;
 
 
-            if ((crate->descr_flags & LTR_CRATE_DESCR_FLAGS_HAS_DATA_CLOSE_BUG) &&
-                    (++intf->data_recon_cnt <= LTR_TCP_DATA_RECON_LIMIT)) {
-                ltrsrv_log_rec(LTRSRV_LOGLVL_WARN, crate->log_str, 0,
-                               "Data socket closed by crate. Start data socket reconnection (reconnection number = %d)!",
-                               intf->data_recon_cnt);
-                err = f_sock_start_conn(&intf->data_sock, AF_INET, crate->par.location,
-                                      LTR_TCP_DATA_PORT, &wt, crate, f_data_cb);
-                if (!err) {
-                    if (wt) {
-                        intf->flags |= _ETH_FLAGS_DATA_SOCK_RECONNECTION;
-                    } else {
-                        f_data_reconnection_finish(crate);
-                    }
+            if (LTR_CRATE_SUPPORT_SEC_CTL_CON(crate)) {
+                if (!(intf->flags & ETH_FLAGS_DATA_SOCK_CHECK_SEC_CON)) {
+                    intf->flags |= ETH_FLAGS_DATA_SOCK_CHECK_SEC_CON;
+                    err = ltrsrv_crate_add_cmd_to_queue(crate,
+                                                        LTR_CRATE_CMD_CODE_READ,
+                                                        LTR_CRATE_CMD_REASON_INTF_REQ,
+                                                        0,
+                                                        LTR_CRATE_ADDR_CONNECTION_PRIMARY,
+                                                        0,
+                                                        LTR_CRATE_REG_CONNECTION_PRIMARY_SIZE,
+                                                        &intf->work_cmd_ctx.recvd_size,
+                                                        intf->work_cmd_ctx.recvd_data,
+                                                        LTR_CRATE_CMD_FLAGS_EXTERNAL_BUF,
+                                                        f_check_sec_con_data_close, NULL);
                 }
             } else {
-                ltrsrv_log_rec(LTRSRV_LOGLVL_WARN, crate->log_str, 0,
-                               "Crate data socket closed by crate");
-                ltrsrv_crate_close_req(crate, 0);
-                err = 0;
+                err = f_start_data_recon(crate);
             }
         } else if (!err) {
 #ifdef LTRD_ETH_DUMP_ENABLED
@@ -921,7 +1011,7 @@ static void f_data_cb(t_socket sock, int event, void* data) {
             /* если появился новый хвост - сохраняем его */
             intf->rcv_data_offs = received % sizeof(uint32_t);
             if (intf->rcv_data_offs) {
-                memcpy(&intf->rcv_last_wrd, &f_rcv_buf[received-intf->rcv_data_offs-1],
+                memcpy(&intf->rcv_last_wrd, &f_rcv_buf[received-intf->rcv_data_offs],
                        intf->rcv_data_offs);
             }
 
@@ -933,7 +1023,7 @@ static void f_data_cb(t_socket sock, int event, void* data) {
     /* готовность на передачу приходит либо если мы делали реконнект сокета, либо
        если нас попросил верхний уровень (в этом случае вызываем его callback) */
     if (!err && (event & LTRSRV_EVENT_SEND_RDY)) {
-        if (intf->flags & _ETH_FLAGS_DATA_SOCK_RECONNECTION) {
+        if (intf->flags & ETH_FLAGS_DATA_SOCK_RECONNECTION) {
             err = ltrsrv_sock_con_done_check(sock);
             if (err) {
                 f_print_connection_err(crate, err, "Data socket connection failed");
@@ -1654,6 +1744,17 @@ int ltrsrv_eth_ip_set_flags(uint32_t addr, uint32_t flags, int permanent, int mo
 }
 
 
+static int f_update_sock_nodelay(t_ltr_crate *crate, void *data) {
+    if (crate->par.intf == LTR_CRATE_IFACE_TCPIP) {
+        if (crate->par.state != LTR_CRATE_CONSTATE_CLOSED) {
+            t_eth_data *intf = (t_eth_data *)crate->intf_data;
+            f_sock_set_nodelay(intf->cmd_sock, f_eth_nodelay);
+            f_sock_set_nodelay(intf->data_sock, f_eth_nodelay);
+        }
+    }
+    return 0;
+}
+
 /***************************************************************************//**
   Установка параметров интерфейса ethernet
     @param[in] param   Параметр из #t_ltrd_eth_params
@@ -1661,16 +1762,17 @@ int ltrsrv_eth_ip_set_flags(uint32_t addr, uint32_t flags, int permanent, int mo
     @param[in] size    Размер значения
     @return            Код ошибки
  ******************************************************************************/
-int ltrsrv_eth_set_param(t_ltr_server_params param, const void *val, uint32_t size) {
+int ltrsrv_eth_set_param(t_ltrd_params param, const void *val, uint32_t size) {
     int err = 0;
-    uint32_t* uintval = (uint32_t*)val;
+    uint32_t uintval;
     switch (param) {
-        case LTR_SERVER_PARAM_ETH_CRATE_POLL_TIME:
-            if (size==sizeof(*uintval)) {
+        case LTRD_PARAM_ETH_CRATE_POLL_TIME:
+            err = ltrsrv_param_get_uint32(val, size, &uintval);
+            if (!err) {
                 int i;
 
                 /* при смене таймаута опроса перезапускаем таймеры всех крейтов */
-                f_eth_poll_time = *uintval;
+                f_eth_poll_time = uintval;
                 if ((f_eth_poll_time!=0) && (f_eth_poll_time< LTRSRV_ETH_CRATE_POLL_INTERVAL_MIN))
                     f_eth_poll_time = LTRSRV_ETH_CRATE_POLL_INTERVAL_MIN;
 
@@ -1683,29 +1785,26 @@ int ltrsrv_eth_set_param(t_ltr_server_params param, const void *val, uint32_t si
                         f_ip_entries[i].crate->intf_func.poll_time = f_eth_poll_time;
                     }
                 }
-            } else {
-                err = LTRSRV_ERR_INVALID_PARAM_SIZE;
             }
             break;
-        case LTR_SERVER_PARAM_ETH_CRATE_CON_TOUT:
-            if (size==sizeof(*uintval)) {
-                f_eth_con_timeout = MAX(*uintval, LTRSRV_ETH_CONNECTION_TOUT_MIN);
+        case LTRD_PARAM_ETH_CRATE_CON_TOUT:
+            err = ltrsrv_param_get_uint32(val, size, &uintval);
+            if (!err) {
+                f_eth_con_timeout = MAX(uintval, LTRSRV_ETH_CONNECTION_TOUT_MIN);
                 ltrsrv_settings_mark_modified();
-            } else {
-                err = LTRSRV_ERR_INVALID_PARAM_SIZE;
             }
             break;
-        case LTR_SERVER_PARAM_ETH_CRATE_CTLCMD_TOUT:
-            if (size==sizeof(*uintval)) {
-                f_eth_cmd_timeout = MAX(*uintval, LTRSRV_ETH_CTL_CMD_TOUT_MIN);
+        case LTRD_PARAM_ETH_CRATE_CTLCMD_TOUT:
+            err = ltrsrv_param_get_uint32(val, size, &uintval);
+            if (!err) {
+                f_eth_cmd_timeout = MAX(uintval, LTRSRV_ETH_CTL_CMD_TOUT_MIN);
                 ltrsrv_settings_mark_modified();
-            } else {
-                err = LTRSRV_ERR_INVALID_PARAM_SIZE;
             }
             break;
-        case LTR_SERVER_PARAM_ETH_INTF_CHECK_TIME:
-            if (size == sizeof(*uintval)) {
-                f_eth_intf_check_time = *uintval;
+        case LTRD_PARAM_ETH_INTF_CHECK_TIME:
+            err = ltrsrv_param_get_uint32(val, size, &uintval);
+            if (!err) {
+                f_eth_intf_check_time = uintval;
                 ltrsrv_settings_mark_modified();
                 if ((f_eth_intf_check_time != 0) &&
                         (f_eth_intf_check_time < LTR_SERVER_DEFAULT_ETH_INTF_CHECK_TIME_MIN)) {
@@ -1718,13 +1817,12 @@ int ltrsrv_eth_set_param(t_ltr_server_params param, const void *val, uint32_t si
                 } else {
                     ltrsrv_wait_remove_tmr(&f_hostaddrs);
                 }
-            } else {
-                err = LTRSRV_ERR_INVALID_PARAM_SIZE;
             }
             break;
-        case LTR_SERVER_PARAM_ETH_RECONNECTION_TIME:
-            if (size == sizeof(*uintval)) {
-                f_eth_reconnect_time = *uintval;
+        case LTRD_PARAM_ETH_RECONNECTION_TIME:
+            err = ltrsrv_param_get_uint32(val, size, &uintval);
+            if (!err) {
+                f_eth_reconnect_time = uintval;
                 ltrsrv_settings_mark_modified();
                 if ((f_eth_reconnect_time != 0) &&
                         (f_eth_reconnect_time < LTRSRV_ETH_RECONNECT_TOUT_MIN)) {
@@ -1737,8 +1835,17 @@ int ltrsrv_eth_set_param(t_ltr_server_params param, const void *val, uint32_t si
                 } else {
                     ltrsrv_wait_remove_tmr(&f_eth_reconnect_time);
                 }
-            } else {
-                err = LTRSRV_ERR_INVALID_PARAM_SIZE;
+            }
+            break;
+    case LTRD_PARAM_ETH_SEND_NODELAY:
+            err = ltrsrv_param_get_uint32(val, size, &uintval);
+            if (!err) {
+                int setval = !!uintval;
+                if (setval != f_eth_nodelay) {
+                    f_eth_nodelay = setval;
+                    ltrsrv_settings_mark_modified();
+                    err = ltrsrv_crates_exec_for_all(f_update_sock_nodelay, NULL);
+                }
             }
             break;
         default:
@@ -1746,27 +1853,11 @@ int ltrsrv_eth_set_param(t_ltr_server_params param, const void *val, uint32_t si
             break;
     }
 
-    if (err) {
-        ltrsrv_log_rec(LTRSRV_LOGLVL_ERR, LTRSRV_LOGSRC_SERVER,err,
-                       "Cannot set " LTRD_PROGRAMM_NAME " ethernet param %d", param);
-    }
-
     return err;
 }
 
 
 
-static LINLINE int f_get_uint32_val(uint32_t set_val, void *val, uint32_t* size) {
-    int err = 0;
-    if (*size>=sizeof(set_val))  {
-        *((uint32_t*)val) = set_val;
-        *size = sizeof(set_val);
-    } else {
-        err = LTRSRV_ERR_INVALID_PARAM_SIZE;
-    }
-    return err;
-}
-
 
 /***************************************************************************//**
   Установка параметров интерфейса ethernet
@@ -1775,34 +1866,32 @@ static LINLINE int f_get_uint32_val(uint32_t set_val, void *val, uint32_t* size)
     @param[in] size    Размер значения
     @return            Код ошибки
  ******************************************************************************/
-int ltrsrv_eth_get_param(t_ltr_server_params param, void *val, uint32_t *size) {
+int ltrsrv_eth_get_param(t_ltrd_params param, void *val, uint32_t *size) {
     int err = 0;
     switch (param) {
-        case LTR_SERVER_PARAM_ETH_CRATE_POLL_TIME:
-            err = f_get_uint32_val(f_eth_poll_time, val, size);
+        case LTRD_PARAM_ETH_CRATE_POLL_TIME:
+            err = ltrsrv_param_put_uint32(val, size, f_eth_poll_time);
             break;
-        case LTR_SERVER_PARAM_ETH_CRATE_CON_TOUT:
-            err = f_get_uint32_val(f_eth_con_timeout, val, size);
+        case LTRD_PARAM_ETH_CRATE_CON_TOUT:
+            err = ltrsrv_param_put_uint32(val, size, f_eth_con_timeout);
             break;
-        case LTR_SERVER_PARAM_ETH_CRATE_CTLCMD_TOUT:
-            err = f_get_uint32_val(f_eth_cmd_timeout, val, size);
+        case LTRD_PARAM_ETH_CRATE_CTLCMD_TOUT:
+            err = ltrsrv_param_put_uint32(val, size, f_eth_cmd_timeout);
             break;
-        case LTR_SERVER_PARAM_ETH_INTF_CHECK_TIME:
-            err = f_get_uint32_val(f_eth_intf_check_time, val, size);
+        case LTRD_PARAM_ETH_INTF_CHECK_TIME:
+            err = ltrsrv_param_put_uint32(val, size, f_eth_intf_check_time);
             break;
-        case LTR_SERVER_PARAM_ETH_RECONNECTION_TIME:
-            err = f_get_uint32_val(f_eth_reconnect_time, val, size);
+        case LTRD_PARAM_ETH_RECONNECTION_TIME:
+            err = ltrsrv_param_put_uint32(val, size, f_eth_reconnect_time);
+            break;
+        case LTRD_PARAM_ETH_SEND_NODELAY:
+            err = ltrsrv_param_put_uint32(val, size, (uint32_t)f_eth_nodelay);
             break;
         default:
             err = LTRSRV_ERR_INVALID_PARAM;
             break;
     }
 
-    if (err) {
-        ltrsrv_log_rec(LTRSRV_LOGLVL_ERR, LTRSRV_LOGSRC_SERVER,err,
-                       "Cannot get " LTRD_PROGRAMM_NAME " ethernet param %d", param);
-    }
-
     return err;
 }
 
@@ -1989,8 +2078,8 @@ int ltrsrv_eth_init(void) {
 
 int ltrsrv_eth_close(void) {
 #ifdef _WIN32
-    free(f_hostaddrs[0]);
-    free(f_hostaddrs[1]);
+    free(f_hostaddrs[0]); f_hostaddrs[0] = NULL;
+    free(f_hostaddrs[1]); f_hostaddrs[1] = NULL;
 #endif
 
 #ifdef HAVE_IFADDRS_H
