@@ -181,7 +181,7 @@ static void f_check_cleanup(t_usb_data* intf) {
     }
 
     /* если все передачи завершены, то можем закрывать описатель */
-    if (!not_free_transfs) {
+    if (not_free_transfs == 0) {
         intf->port_intf->cleanup(intf);
     }
 }
@@ -226,6 +226,10 @@ static void f_check_disc_cb(t_socket sock, int event, void* data) {
                     ltrsrv_log_syserr(intf->crate->log_str, intf->cur_err,
                                       intf->cur_syserr, "Crate data send process error");
                     break;
+                case USB_CLOSE_REASON_BULK_OUT_OF_ORDER:
+                    ltrsrv_log_syserr(intf->crate->log_str, intf->cur_err,
+                                      intf->cur_syserr, "Crate transfer out of order complete");
+                    break;
                 default:
                     break;
 
@@ -582,7 +586,7 @@ int p_usb_init_new_dev(void *devid, const t_usb_port_intf *port_intf) {
         err = LTRSRV_ERR_NO_FREE_CRATE_SLOT;
         port_intf->devid_free(devid);
     } else {
-        t_ltr_crate* crate = ( t_ltr_crate*)calloc(1, sizeof(t_ltr_crate));
+        t_ltr_crate *crate = ( t_ltr_crate*)calloc(1, sizeof(t_ltr_crate));
         t_usb_data *intf = (t_usb_data *)calloc(1, sizeof(t_usb_data));
 
         if ((crate!=NULL) && (intf!=NULL)) {
@@ -645,7 +649,7 @@ void p_usb_check_devlist_finish(void) {
  * запланирована на время завершения всех трансферов */
 static void f_usb_free_data(t_usb_data* intf, t_usb_close_reasons reason) {
     /* удаляем крейт из сестемы, если зарегистрирован */
-    if (reason == USB_CLOSE_REASON_DISCONNECT) {
+    if ((reason == USB_CLOSE_REASON_DISCONNECT) || (reason == USB_CLOSE_REASON_CRATE_REQ)) {
         f_usb_unregister_crate(intf);
     }
 
@@ -677,6 +681,8 @@ void p_usb_close_dev_by_err(t_usb_data* intf, t_usb_close_reasons reason, int er
     if (!intf->close_reason) {
        f_set_dev_err(intf, err, syserr);
        f_usb_free_data(intf, reason);
+    } else {
+        f_check_cleanup(intf);
     }
 }
 
@@ -912,15 +918,20 @@ int p_usb_process_ioctl(t_usb_data *intf, int transf_err, const uint8_t *reply,
             ltrsrv_crate_register(intf->crate);
         } else {
             /* остальные ошибки считаем фатальными */
-            err = LTRSRV_ERR_USB_TRANSFER_FAILED;
+            err = transf_err;
 
             /* при ошибке в рабочем режиме, возвращаем ошибку наверхнем уровне.
              * а при инициализации - удаляем крейт сразу */
             if (intf->crate->par.state == LTR_CRATE_CONSTATE_WORK) {
-                ltrsrv_log_rec(LTRSRV_LOGLVL_ERR, intf->crate->log_str, err,
-                               "control transfer failed");
-                if (intf->proc_cmd_cb)
-                    intf->proc_cmd_cb(intf->crate, LTRSRV_ERR_USB_TRANSFER_FAILED, intf->proc_data);
+                if (intf->proc_cmd_cb) {
+                    intf->proc_cmd_cb(intf->crate, err, intf->proc_data);
+                } else {
+                    /* если есть cb, то вывод об ошибке будет в нем на более высоком
+                     * уровне и нет смысла дублировать, поэтому выводим сообщение
+                     * тут только при отсутствии cb */
+                    ltrsrv_log_rec(LTRSRV_LOGLVL_ERR, intf->crate->log_str, err,
+                                   "control transfer failed");
+                }
             } else {
                 ltrsrv_log_rec(LTRSRV_LOGLVL_ERR, intf->crate->log_str, err,
                                "Crate USB initialization stage '%s' control request failed",
@@ -1142,6 +1153,14 @@ int p_usb_process_bulk_wr(t_usb_data *intf, int transf_err, int syserr, uint8_t
 
 
 
+void p_usb_process_bulk_out_of_order(t_usb_data *intf, t_transf_params *transf, int transf_err, int syserr) {
+   transf->state = USB_TRANSF_STATE_FREE;
+   p_usb_close_dev_by_err(intf, USB_CLOSE_REASON_BULK_OUT_OF_ORDER,
+                          transf_err ? transf_err : LTRSRV_ERR_USB_TRANSF_OUT_OF_ORDER, syserr);
+
+}
+
+
 int p_usb_close(void) {
     int i;
     /* отменяем инициализацию всех крейтов, которые не дошли до рабочего состояния */
@@ -1150,5 +1169,3 @@ int p_usb_close(void) {
     }
     return 0;
 }
-
-
