@@ -169,7 +169,7 @@ FILE* f_fopen_spec(const char* fname, const char* mode) {
 static void f_ioctl_cb(t_socket sock, int event, void* data) {
     int trans_err = 0;
     int proc = 0;
-    DWORD len;
+    DWORD len = 0;
     t_usb_data *intf = (t_usb_data *)data;
     t_usb_lcomp_data *privdata = (t_usb_lcomp_data *)intf->privdata;
 
@@ -198,7 +198,7 @@ static void f_ioctl_cb(t_socket sock, int event, void* data) {
 static void f_bulk_cb(t_socket sock, int event, void* data) {
     int transf_err = 0;
     int proc = 0;
-    DWORD len;
+    DWORD len = 0;
     t_usb_data *intf = (t_usb_data *)data;
     t_usb_lcomp_data *privdata = (t_usb_lcomp_data *)intf->privdata;
     t_transf_params *transf;
@@ -220,9 +220,10 @@ static void f_bulk_cb(t_socket sock, int event, void* data) {
             proc = 1;            
             if(!GetOverlappedResult(privdata->hDevice, pOv, &len, TRUE)) {
                 syserr = LTRSRV_LAST_SYSERR;
-                if (!privdata->canceled)
-                    CancelIo(privdata->hDevice);
-                transf_err = LTRSRV_ERR_USB_TRANSFER_FAILED;
+                if (!privdata->canceled) {
+                    if (syserr != ERROR_OPERATION_ABORTED)
+                        transf_err = LTRSRV_ERR_USB_TRANSFER_FAILED;
+                }
             }
         }
 
@@ -247,9 +248,11 @@ static void f_bulk_cb(t_socket sock, int event, void* data) {
                 proc = 1;
                 if(!GetOverlappedResult(privdata->hDevice, pOv, &len, TRUE)) {
                     syserr = LTRSRV_LAST_SYSERR;
-                    if (!privdata->canceled)
+                    if (!privdata->canceled) {
                         CancelIo(privdata->hDevice);
-                    transf_err = LTRSRV_ERR_USB_TRANSFER_FAILED;
+                        if (syserr != ERROR_OPERATION_ABORTED)
+                            transf_err = LTRSRV_ERR_USB_TRANSFER_FAILED;
+                    }
                 }
             }
             CloseHandle(pOv->hEvent);
