@@ -270,18 +270,42 @@ static int f_get_transf_err(struct libusb_transfer *transfer) {
     return 0;
 }
 
+/* если обмен завершился не по очереди, то находим соответствующий элемент в поставленных
+ * на передачу запросах и завершаем его через p_usb_process_bulk_out_of_order */
+static void f_proc_out_of_order_transf(t_bulk_transf_ctl *ctl, struct libusb_transfer *transfer) {
+    int pos = ctl->get_pos;
+    int out = 0;
+    t_usb_data* intf = (t_usb_data*)transfer->user_data;
+
+    while (!out) {
+        t_transf_params *transf_par = &intf->bulk_rd.transfs[pos];
+        if (transf_par->state != USB_TRANSF_STATE_FREE) {
+            if (transf_par->port_transfer == transfer) {
+                int err = f_get_transf_err(transfer);
+                int syserr = (err == LTRSRV_ERR_USB_TRANSFER_FAILED) ? LTRSRV_LAST_SYSERR : 0;
+                p_usb_process_bulk_out_of_order(intf, transf_par, err, syserr);
+                out = 1;
+            } else {
+                if (++pos==LTRSRV_USB_BULK_MAX_TRANSF_CNT)
+                    pos = 0;
+                if (pos == ctl->put_pos)
+                    out = 1;
+            }
+        }
+    }
+}
+
 
 /* Callback на завершения приема данных */
 static void f_cb_rd(struct libusb_transfer *transfer) {
     t_usb_data* intf = (t_usb_data*)transfer->user_data;
 
     if (intf->bulk_rd.transfs[intf->bulk_rd.get_pos].port_transfer != transfer) {
-        ltrsrv_log_rec(LTRSRV_LOGLVL_ERR_FATAL, intf->crate->log_str,
-                       LTRSRV_ERR_USB_TRANSF_OUT_OF_ORDER, "Invalid recv transfer completed");
+        f_proc_out_of_order_transf(&intf->bulk_rd, transfer);
     } else {
         int err = f_get_transf_err(transfer);
         int syserr = (err == LTRSRV_ERR_USB_TRANSFER_FAILED) ? LTRSRV_LAST_SYSERR : 0;
-        p_usb_process_bulk_rd(intf, err, syserr, transfer->buffer, transfer->actual_length);
+        p_usb_process_bulk_rd(intf, err, syserr, transfer->buffer, (unsigned)transfer->actual_length);
     }
 }
 
@@ -292,12 +316,11 @@ static void f_cb_wr(struct libusb_transfer *transfer) {
     t_usb_data* intf = (t_usb_data*)transfer->user_data;
 
     if (intf->bulk_wr.transfs[intf->bulk_wr.get_pos].port_transfer != transfer) {
-        ltrsrv_log_rec(LTRSRV_LOGLVL_ERR_FATAL, intf->crate->log_str,
-                       LTRSRV_ERR_USB_TRANSF_OUT_OF_ORDER, "Invalid send transfer completed");
+        f_proc_out_of_order_transf(&intf->bulk_wr, transfer);
     } else {
         int err = f_get_transf_err(transfer);
         int syserr = (err == LTRSRV_ERR_USB_TRANSFER_FAILED) ? LTRSRV_LAST_SYSERR : 0;
-        p_usb_process_bulk_wr(intf, err, syserr, transfer->buffer, transfer->actual_length);
+        p_usb_process_bulk_wr(intf, err, syserr, transfer->buffer, (unsigned)transfer->actual_length);
     }
 }
 
