@@ -24,12 +24,12 @@
 
 
 typedef struct {
-    void* handle;
+    void *handle;
     t_socket sock;
     int flags; /* флаги из t_ltrsrv_wait_flags */
     t_ltimer tmr;
     t_ltrsrv_wait_cb cb;
-    void* data;
+    void *data;
     int event; /* флаги из t_ltrsrv_wait_event, определяющие, какое событие произошло */
 #ifdef _WIN32
     HANDLE hEvent;
@@ -79,13 +79,13 @@ static void f_delete_socks(void) {
     }
 }
 
-int ltrsrv_wait_add_tmr(const void* hnd, int flags, unsigned int tout,
-                        t_ltrsrv_wait_cb cb, void* data) {
+int ltrsrv_wait_add_tmr(const void *hnd, int flags, unsigned int tout,
+                        t_ltrsrv_wait_cb cb, void *data) {
     int i, fnd;
     int err = 0;
     t_lclock_ticks tmr_clocks = LTIMER_MS_TO_CLOCK_TICKS(tout);
     /* проверяем - не зарегисрирован ли уже этот сокет */
-    for (i=0, fnd=0; (i < f_wt_sock_cnt) && !fnd; i++) {
+    for (i = 0, fnd = 0; (i < f_wt_sock_cnt) && !fnd; ++i) {
         if ((f_wt_socks[i].handle == hnd) &&
                 (f_wt_socks[i].flags & LTRSRV_WAITFLG_TIMER_HANDLE) &&
                 !(f_wt_socks[i].flags & LTRSRV_WAITFLG_DELETE))
@@ -283,7 +283,7 @@ static void f_wait_call_cb(void) {
                события. устанавливаем до callback'a, т.к. там может сокет быть
                заново зарегистрирован*/
             if (f_wt_socks[i].flags & LTRSRV_WAITFLG_AUTODEL) {
-                f_wt_socks[i].flags |= LTRSRV_WAITFLG_DELETE;                
+                f_wt_socks[i].flags |= LTRSRV_WAITFLG_DELETE;
             }
             f_wt_socks[i].cb(f_wt_socks[i].sock, f_wt_socks[i].event, f_wt_socks[i].data);
             f_wt_socks[i].event = 0;
@@ -550,7 +550,7 @@ int ltrsrv_addr_to_string(const struct sockaddr *saddr, size_t addr_len, char* s
 #ifdef _WIN32
     int err = WSAAddressToStringA((struct sockaddr *)saddr, addr_len, NULL, str, &str_len) ? LTRSRV_ERR_SOCKET_INET_ADDR : 0;
     if (!err)
-        str[str_len]=0;
+        str[str_len-1]=0;
     return err;
 #else
     void* addr=NULL;
